/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/


#ifndef _GB2_SERIALIZE_UTILS_H_
#define _GB2_SERIALIZE_UTILS_H_

#include <core_api/core_api.h>

namespace GB2 {

class DNASequence;
class RemoteMachineSettings;
class RemoteMachine;
class MAlignment;
class DNAAlphabet;

class GB2_COREAPI_EXPORT SerializeUtils {
private:
    static const int SERIALIZED_DNASEQUENCE_LIST_SZ = 3;
    static const int SERIALIZED_MALIGNMENT_LIST_SZ = 3;
    
    static const QString REMOTE_MACHINE_VERSION_1_0;
    
public:
    template<class T> static QVariant serializeArray( T * arr, int sz ) {
        if( NULL == arr ) {
            return 0;
        }
        assert( 0 < sz );
        QVariantList res;
        for( int i = 0; i < sz; ++i ) {
            res << arr[i];
        }
        return res;
    }

    /* the arr array must be allocated by caller */
    template<class T> static bool deserializeArray( const QVariant & data, T * arr, int sz ) {
        if( data.canConvert( QVariant::Int ) && 0 == data.toInt() ) {
            return true;
        }
        if( 0 >= sz ) {
            return false;
        }
        if( !data.canConvert( QVariant::List ) ) {
            return false;
        }
        QVariantList args = data.toList();
        if( args.size() != sz ) {
            return false;
        }

        for( int i = 0; i < sz; ++i ) {
            const QVariant & currentArg = args[i];
            if( !currentArg.canConvert<T>() ) {
                return false;
            }
            arr[i] = currentArg.value<T>();
        }
        return true;
    }

    static QVariant serializeString( const char * data );
    static bool deserializeString( const QVariant & data, QByteArray * arr );

    static QVariant serializeDNASequence( const DNASequence & seq );
    static bool deserializeDNASequence( const QVariant & data, DNASequence * seq );
    
    static QVariant serializeDNAAlphabet( DNAAlphabet * al );
    static bool deserializeDNAAlphabet( const QVariant & data, DNAAlphabet ** al );
    
    static QVariant serializeMAlignment( const MAlignment & ma );
    static bool deserializeMAlignment( const QVariant & data, MAlignment * ma );
    
    template<class T> static bool deserializeValue( const QVariant & data, T * val ) {
        if( NULL == val ) {
            return false;
        }
        if( !data.canConvert<T>() ) {
            return false;
        }
        *val = data.value<T>();
        return true;
    }
    
    static QString serializeRemoteMachineSettings( RemoteMachineSettings * machine );
    static bool deserializeRemoteMachineSettings( const QString & data, RemoteMachineSettings ** settings, QString * protoId = NULL );
    static bool deserializeRemoteMachineSettings( const QString & data, RemoteMachine ** machine );
    static bool deserializeRemoteMachineSettingsFromFile( const QString & machinePath, RemoteMachine ** machine ); /* needed for tests */
    
}; // SerializeUtils

} // GB2

#endif // _GB2_SERIALIZE_UTILS_H_
