/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "CreatePhyTreeDialogController.h"
#include "PhyTreeGeneratorRegistry.h"

#include <util_gui/DialogUtils.h>
#include <document_format/GenbankLocationParser.h>
#include <core_services/DocumentFormatRegistryImpl.h>
#include <core_api/AppContext.h>
#include <core_api/IOAdapter.h>
#include <core_api/SubstMatrixRegistry.h>

#include <QtGui/qfiledialog.h>
#include <QtGui/qmessagebox.h>

namespace GB2{

CreatePhyTreeDialogController::CreatePhyTreeDialogController(const MAlignment& _mobj, CreatePhyTreeSettings& _settings)
: QDialog(NULL), mobj(_mobj), settings(_settings){
    setupUi(this);
    verticalLayout->getContentsMargins ( NULL, NULL, &rightMargin, NULL );
    PhyTreeGeneratorRegistry* registry = AppContext::getPhyTreeGeneratorRegistry();
    QStringList nameList = registry->getNameList();
    for(int i=0; i<nameList.length(); i++){
        algorithmBox->addItem(nameList[i]);
    }
    PhyTreeGenerator* generator = registry->getGenerator(nameList.at(0));
    generator->setupCreatePhyTreeUI(this, _mobj);
    verticalLayout->activate();
}


void CreatePhyTreeDialogController::accept(){

    settings.algorithmId = algorithmBox->currentText();
    foreach (CreatePhyTreeWidget* widget, childWidgets) {
        widget->fillSettings(settings);
    }

    QDialog::accept();

}

void CreatePhyTreeDialogController::insertWidget( int pos, CreatePhyTreeWidget* widget )
{
    verticalLayout->insertWidget(pos,widget);
    childWidgets.append(widget);
    
    // adjust sizes
    setMinimumHeight(widget->minimumHeight() + minimumHeight());
    if (minimumWidth() < widget->minimumWidth()) {
        setMinimumWidth(widget->minimumWidth() + rightMargin*2);
    };
    widget->show();
}

};
