/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "CreateSubalignimentTest.h"

#include <core_api/DocumentModel.h>

namespace GB2{

#define DOC_ATTR "doc_name"
#define SEQ_NAMES_ATTR "sequences"
#define WINDOW_ATTR "window"
#define EXPECTED_DOC_ATTR "expected_doc_name"

void GTest_CreateSubalignimentTask::init(XMLTestFormat *tf, const QDomElement& el){
	Q_UNUSED(tf);
	QString buf = el.attribute(DOC_ATTR);
	if(buf.isEmpty()){
		stateInfo.setError(GTest::tr("value not set %1").arg(DOC_ATTR));
		return;
	}
	docName = buf;

	buf = el.attribute(EXPECTED_DOC_ATTR);
	if(buf.isEmpty()){
		stateInfo.setError(GTest::tr("value not set %1").arg(EXPECTED_DOC_ATTR));
		return;
	}
	expectedDocName = buf;

	buf = el.attribute(SEQ_NAMES_ATTR);
	if(buf.isEmpty()){
		stateInfo.setError(GTest::tr("value not set %1").arg(SEQ_NAMES_ATTR));
		return;
	}
	seqNames = buf.split(",", QString::SkipEmptyParts);
	if (seqNames.isEmpty())	{
		stateInfo.setError(GTest::tr("no subsequences selected"));
		return;
	}

	buf = el.attribute(WINDOW_ATTR);
	if(buf.isEmpty()){
		stateInfo.setError(GTest::tr("value not set %1").arg(WINDOW_ATTR));
		return;
	}

	QStringList bufList = buf.split("..");
	if(bufList.size() != 2){
		stateInfo.setError(GTest::tr("invalid region %1").arg(WINDOW_ATTR));
		return;
	}
	int start, end;
	bool conversion;
	start = bufList.first().toInt(&conversion);
	if(!conversion){
		stateInfo.setError(GTest::tr("start position of window not an integer"));
		return;
	}

	end = bufList.last().toInt(&conversion);
	if(!conversion){
		stateInfo.setError(GTest::tr("end position of window not an integer"));
		return;
	}

	window = LRegion(start, end - start);
}

void GTest_CreateSubalignimentTask::prepare(){
	Document* doc = getContext<Document>(this, docName);
	if (doc == NULL) {
		stateInfo.setError(GTest::tr("context not found %1").arg(docName));
		return;
	}

	QList<GObject*> list = doc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
	if (list.size() == 0) {
		stateInfo.setError(GTest::tr("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT));
		return;
	}

	Document* expectedDoc = getContext<Document>(this, expectedDocName);
	if (doc == NULL) {
		stateInfo.setError(GTest::tr("context not found %1").arg(expectedDocName));
		return;
	}

	QList<GObject*> expList = expectedDoc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
	if (list.size() == 0) {
		stateInfo.setError(GTest::tr("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT));
		return;
	}

	expectedMaobj = (MAlignmentObject*)expList.first();

	maobj = (MAlignmentObject*)list.first();
	t = new CreateSubalignimentTask(maobj, window, seqNames, doc->getURL(), false, false);
	addSubTask(t);
}

Task::ReportResult GTest_CreateSubalignimentTask::report(){
	MAlignment actual = maobj->getMAlignment(),
				expected = expectedMaobj->getMAlignment();
	if(actual.alignedSeqs.size() != expected.alignedSeqs.size()){
		stateInfo.setError(GTest::tr("Expected and actual aligniment sizes are different: %1 , %2")
			.arg(expected.alignedSeqs.size())
			.arg(actual.alignedSeqs.size()));
		return ReportResult_Finished;
	}
	for(int i = 0; i < actual.alignedSeqs.size(); i++){
		MAlignmentItem actItem = actual.alignedSeqs.at(i), expItem = expected.alignedSeqs.at(i);
		if (actItem != expItem){
			stateInfo.setError(GTest::tr("Expected and actual aligniments not equal"));
			return ReportResult_Finished;
		}
	}
	return ReportResult_Finished;
}

QList< XMLTestFactory* > CreateSubalignimentTests::createTestFactories(){
	QList< XMLTestFactory* > res;
	res.append( GTest_CreateSubalignimentTask::createFactory() );
	return res;
}

}
