/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_MSA_EDITOR_TREE_VIEWER_H_
#define _GB2_MSA_EDITOR_TREE_VIEWER_H_

#include <QtGui/QGraphicsView>
#include <QtGui/QToolBar>
#include <QtGui/QToolButton>
#include <QtGui/QScrollBar>
#include <QtCore/QMap>
#include <datatype/MAlignment.h>
#include <gobjects/MAlignmentObject.h>
#include <gobjects/PhyTreeObject.h>
#include <datatype/PhyTree.h>
#include <core_api/Task.h>
#include <phyltree/PhyTreeGeneratorTask.h>
#include <phyltree/CreatePhyTreeSettings.h>

namespace GB2 {

class GObjectView;
class MSAEditor;
class MSAEditorUI;
class MAlignmentItem;
class GraphicsBranchItem;

class MSAEditorTreeViewer: public QGraphicsView {
    Q_OBJECT
public:
    MSAEditorTreeViewer(MSAEditorUI* ui, QScrollBar *vb);
    virtual ~MSAEditorTreeViewer();
    void setTree(PhyTreeObject* _tree);
    bool isCreated() const { return treeObj != NULL; }

    static int initialWidth;

protected:
    virtual void wheelEvent(QWheelEvent *e);
    virtual void resizeEvent(QResizeEvent *e);
    virtual void mousePressEvent(QMouseEvent *e);

private slots:
    void sl_buildStaticToolbar(GObjectView* v, QToolBar* tb);
    void sl_sortTriggered();
    void sl_printTriggered();
    void sl_captureScreenTriggered();
    void sl_captureTreeTriggered();
    void sl_showTreeTriggered(bool on);
    void sl_showSeqsTriggered(bool on);
    void sl_buildTreeTriggered();
    void sl_startChanged(const QPoint& p, const QPoint& prev);
    void sl_alignmentChanged(const MAlignment& beforMA, const MAlignmentModInfo& info);
    void sl_viewRecomputed(GraphicsBranchItem *gbi);
    void sl_recomputeView();
    void sl_showTree();
    void sl_onVScrollMoved(int pos);
    void sl_showNameLabelsTriggered(bool on);
    void sl_showDistanceLabelsTriggered(bool on);
    
private:
    enum LabelType {
        LabelType_SequnceName = 1,
        LabelType_Distance = 2,
    };
    typedef QFlags<LabelType> LabelTypes;

    void updateToolBar();
    void saveImageDialog(QString &fileName, QString &format);
    void fillMAlignment(PhyNode *node, MAlignment &msa);
    void recomputeView(bool treeChanged);
    void widthChanged(qreal w);
    QRect getSceneBoundingRect();
    void paint(QPainter &painter, QRectF rect, qreal k = -2);
    void updateAligns();
    void showLabels(LabelTypes labelTypes);

    QMap<PhyNode*, int> aligns;
    int                 oldScrollPos;
    QByteArray          state;
    QScrollBar*         vBar;
    MSAEditor*          editor;
    MSAEditorUI*        ui;
    GraphicsBranchItem* root;
    CreatePhyTreeSettings settings;
    PhyTreeObject*      treeObj;
    QAction*            buildTreeAction;
    QAction*            showTreeAction;
    QAction*            showSeqsAction;
    QAction*            sortAction;
    QAction*            printAction;
    QAction*            captureTreeAction;
    QAction*            captureScreenAction;
    QAction*            nameLabelsAction;
    QAction*            distanceLabelsAction;
    QAction*            labelsAction;
    PhyTreeGeneratorTask* treeGeneratorTask;
    QString             generatorName;
    QToolButton*        showButton;
    bool                createNewDocument;
    bool                showNameLabels;
    bool                showDistanceLabels;
    int                 maxNameWidth;
};

}//namespace
#endif
